// CloudObject.ts
export class CloudObject {
    className: string;
    id: string | null = null;
    createdAt:any;
    updatedAt:any;
    data: Record<string, any> = {};

    constructor(className: string) {
        this.className = className;
    }

    toPointer() {
        return { "__type": "Pointer", "className": this.className, "objectId": this.id };
    }

    set(json: Record<string, any>) {
        Object.keys(json).forEach(key => {
            if (["objectId", "id", "createdAt", "updatedAt", "ACL"].indexOf(key) > -1) {
                return;
            }
            this.data[key] = json[key];
        });
    }

    get(key: string) {
        return this.data[key] || null;
    }

    async save() {
        let method = "POST";
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}`;

        // 更新
        if (this.id) {
            url += `/${this.id}`;
            method = "PUT";
        }

        const body = JSON.stringify(this.data);
        const response = await fetch(url, {
            headers: {
                "content-type": "application/json;charset=UTF-8",
                "x-parse-application-id": "dev"
            },
            body: body,
            method: method,
            mode: "cors",
            credentials: "omit"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result?.error);
        }
        if (result?.objectId) {
            this.id = result?.objectId;
        }
        return this;
    }

    async destroy() {
        if (!this.id) return;
        const response = await fetch(`http://dev.fmode.cn:1337/parse/classes/${this.className}/${this.id}`, {
            headers: {
                "x-parse-application-id": "dev"
            },
            body: null,
            method: "DELETE",
            mode: "cors",
            credentials: "omit"
        });

        const result = await response?.json();
        if (result) {
            this.id = null;
        }
        return true;
    }
}

// CloudQuery.ts
export class CloudQuery {
    className: string;
    whereOptions: Record<string, any> = {};

    constructor(className: string) {
        this.className = className;
    }

    greaterThan(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$gt"] = value;
    }

    greaterThanAndEqualTo(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$gte"] = value;
    }

    lessThan(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$lt"] = value;
    }

    lessThanAndEqualTo(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$lte"] = value;
    }

    equalTo(key: string, value: any) {
        this.whereOptions[key] = value;
    }

    async get(id: string) {
        const url = `http://dev.fmode.cn:1337/parse/classes/${this.className}/${id}?`;

        const response = await fetch(url, {
            headers: {
                "if-none-match": "W/\"1f0-ghxH2EwTk6Blz0g89ivf2adBDKY\"",
                "x-parse-application-id": "dev"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response?.json();
        return json || {};
    }

    async find() {
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}?`;

        if (Object.keys(this.whereOptions).length) {
            const whereStr = JSON.stringify(this.whereOptions);
            url += `where=${whereStr}`;
        }

        const response = await fetch(url, {
            headers: {
                "if-none-match": "W/\"1f0-ghxH2EwTk6Blz0g89ivf2adBDKY\"",
                "x-parse-application-id": "dev"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response?.json();
        let list = json?.results || []
        let objList = list.map((item:any)=>this.dataToObj(item))
        return objList || [];
    }

    async first() {
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}?`;

        if (Object.keys(this.whereOptions).length) {
            const whereStr = JSON.stringify(this.whereOptions);
            url += `where=${whereStr}`;
        }

        const response = await fetch(url, {
            headers: {
                "if-none-match": "W/\"1f0-ghxH2EwTk6Blz0g89ivf2adBDKY\"",
                "x-parse-application-id": "dev"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response?.json();
        const exists = json?.results?.[0] || null;
        if (exists) {
            let existsObject = this.dataToObj(exists)
            return existsObject;
        }
        return null
    }

    dataToObj(exists:any):CloudObject{
        let existsObject = new CloudObject(this.className);
        existsObject.set(exists);
        existsObject.id = exists.objectId;
        existsObject.createdAt = exists.createdAt;
        existsObject.updatedAt = exists.updatedAt;
        return existsObject;
    }
}